"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const merge = require("lodash.merge");
const resolvers = require("./resolvers");
const util_1 = require("./util");
class TypeConfBase {
    constructor() {
        this.override = {};
        this.stores = {};
        this.rootAccessor = util_1.baseAccessor;
    }
    resolve(name, resolve = _ => _) {
        if (name in this.override) {
            return resolve(this.override[name]);
        }
        return this.rootAccessor(name, resolve);
    }
    exists(name) {
        if (name in this.override) {
            return true;
        }
        return this.rootAccessor(name, true);
    }
    addStore(store, name) {
        this.stores[name] = store;
        this.rootAccessor = util_1.createAccessor(store, this.rootAccessor);
    }
    withStore(storage, name = util_1.randomString()) {
        const store = util_1.createStore(new util_1.ObjectSupplier(storage));
        this.addStore(store, name);
        return this;
    }
    withSupplier(supplier, name = util_1.randomString()) {
        const store = util_1.createStore({
            get: supplier,
            aggregate() {
                return {}; // FIXME
            }
        });
        this.addStore(store, name);
        return this;
    }
    withArgv(_) {
        return this;
    }
    withEnv(_, __) {
        return this;
    }
    withFile(_) {
        return this;
    }
    withDOMNode(_) {
        return this;
    }
    set(key, value) {
        if (typeof value === 'undefined') {
            return this;
        }
        this.override[key] = value;
        return this;
    }
    unset(key) {
        delete this.override[key];
        return this;
    }
    get(name, transform) {
        if (typeof transform === 'function') {
            return transform(this.resolve(name));
        }
        else {
            return this.resolve(name);
        }
    }
    getString(name, fallback) {
        const value = this.resolve(name, resolvers.resolveString);
        if (value !== undefined)
            return value;
        return util_1.assertString(fallback);
    }
    getNumber(name, fallback) {
        const value = this.resolve(name, resolvers.resolveNumber);
        if (value !== undefined)
            return value;
        return util_1.assertNumber(fallback);
    }
    getBoolean(name) {
        const value = this.resolve(name);
        return value !== false && value !== 'false' && this.exists(name);
    }
    getObject(name, fallback) {
        const value = this.resolve(name, resolvers.resolveObject);
        if (value !== undefined)
            return value;
        return util_1.assertObject(fallback);
    }
    getType(name, newable, fallback) {
        const value = this.resolve(name, resolvers.resolveType(newable));
        if (value !== undefined)
            return value;
        return util_1.assertType(fallback, newable);
    }
    toJSON() {
        const aggregate = {};
        for (const store of Object.values(this.stores)) {
            merge(aggregate, store.aggregate());
        }
        return merge(aggregate, this.override);
    }
    toBase64() {
        throw new Error('not implemented: toBase64');
    }
}
exports.default = TypeConfBase;
//# sourceMappingURL=TypeConfBase.js.map