"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const camelCase = require("camel-case");
const constantCase = require("constant-case");
const fs = require("fs");
const path = require("path");
const TypeConfBase_1 = require("./TypeConfBase");
const util_1 = require("./util");
function readFile(file, parser) {
    try {
        const result = parser(fs.readFileSync(file, 'utf8'));
        if (typeof result === 'object' && result !== null)
            return result;
        console.error(`Invalid configuration file: ${file}`);
    }
    catch (e) {
        console.error(`Unable to read ${file}`);
    }
    return {};
}
function readConfigFile(filePath) {
    const extname = path.extname(filePath);
    switch (extname) {
        case '.yml':
        case '.yaml':
            return readFile(filePath, require('js-yaml').safeLoad);
        case '.json':
            return readFile(filePath, JSON.parse);
        default:
            console.error(`Unsupported file: ${filePath}\n`);
            return {};
    }
}
function defaultArgvParser(args) {
    const minimist = require('minimist');
    const _a = minimist(args), { _ } = _a, rest = __rest(_a, ["_"]);
    return rest;
}
class TypeConfNode extends TypeConfBase_1.default {
    withArgv(parser = defaultArgvParser) {
        const argv = parser(process.argv.slice(2));
        const store = util_1.createStore(new util_1.ObjectSupplier(argv), camelCase);
        this.addStore(store, '__argv__');
        return this;
    }
    withEnv(prefix = '', separator = '__') {
        const prefixValue = constantCase(prefix);
        // Transform a configuration value name into an environment variable name.
        const getEnvName = (name) => {
            const keyValue = constantCase(name);
            return prefixValue ? `${prefixValue}_${keyValue}` : keyValue;
        };
        const store = util_1.createStore({
            get(name) {
                const envName = getEnvName(name);
                const result = process.env[envName];
                if (typeof result !== 'undefined')
                    return result;
                const aggregate = util_1.aggregateContainerValues(process.env, envName, separator, camelCase);
                if (Object.keys(aggregate).length > 0) {
                    return aggregate;
                }
            },
            aggregate() {
                // We can only find relevant environment variables if a prefix is specified.
                if (!prefixValue) {
                    return {};
                }
                return util_1.aggregateContainerValues(process.env, prefixValue, separator, camelCase);
            }
        });
        const storeName = prefixValue ? `ENV_${prefixValue}` : 'ENV';
        this.addStore(store, storeName);
        return this;
    }
    withFile(file) {
        if (!file) {
            return this;
        }
        const filePath = path.resolve(process.cwd(), file);
        const storage = readConfigFile(filePath);
        const store = util_1.createStore(new util_1.ObjectSupplier(storage));
        super.addStore(store, filePath);
        return this;
    }
    toBase64() {
        return new Buffer(JSON.stringify(this.toJSON())).toString('base64');
    }
}
exports.default = TypeConfNode;
//# sourceMappingURL=TypeConfNode.js.map