import Newable from './Newable';
export interface Accessor {
    (name: string, peek: true): boolean;
    <T>(name: string, resolve?: Resolver<T>): T | undefined;
    <T>(name: string, arg1?: true | Resolver<T>): boolean | T | undefined;
}
export interface Store {
    get<T>(name: string, next: Accessor, resolve?: Resolver<T>): T | undefined;
    has(name: string, next: Accessor): boolean;
    aggregate(): ObjectStorage;
}
export declare type ObjectStorage = {
    [name: string]: any;
};
export interface Supplier {
    get(name: string): any;
    aggregate(): ObjectStorage;
}
export declare function isSupplier(value: any): value is Supplier;
export declare class ObjectSupplier implements Supplier {
    private readonly storage;
    constructor(storage: ObjectStorage);
    get(name: string): any;
    aggregate(): ObjectStorage;
}
export declare type Resolver<T> = (value?: any) => T | undefined;
export declare function getValueOrNext<T>(name: string, supplier: Supplier, next: Accessor, resolve?: Resolver<T>): T | undefined;
export declare function peekValueOrNext(name: string, supplier: Supplier, next: Accessor): boolean;
export declare function createStore(supplier: Supplier, nameModifier?: (name: string) => string): Store;
export declare function createAccessor(store: Store, next: Accessor): Accessor;
export declare const baseAccessor: Accessor;
export declare type Aggregate = {
    [key: string]: string | Aggregate;
};
export declare function aggregateContainerValues(container: {
    [key: string]: string | undefined;
}, prefix: string, separator: string, keyTransform: (key: string) => string): Aggregate;
export declare function randomString(): string;
export declare function assertString(value: any): string | undefined;
export declare function assertNumber(value: any): number | undefined;
export declare function assertObject(value: any): object | undefined;
export declare function assertType<T>(value: any, newable: Newable<T>): T | undefined;
